@@ -23,7 +23,7 @@
 #include "logview.h"
 #include "calendar.h"
 
-static void calendar_day_selected (GtkWidget *widget, LogviewWindow *window);
+static GtkTreePath *calendar_day_selected (GtkWidget *widget, LogviewWindow *window);
 static void calendar_day_selected_double_click (GtkWidget *widget, LogviewWindow *window);
 static void close_calendar (GtkWidget * widget, int arg, gpointer client_data);
 static DateMark* find_prev_mark (CalendarData*);
@@ -40,71 +40,32 @@ static void calendar_month_changed (GtkWidget *widget, LogviewWindow *window);
 void
 CalendarMenu (LogviewWindow *window)
 {
+	PangoFontDescription *fontdesc;
+	PangoContext *context;
+	GtkCalendar *calendar;
+	int size;
+	
+	calendar = (GtkCalendar *)gtk_calendar_new();
+	
+	g_signal_connect (G_OBJECT (calendar), "month_changed",
+										G_CALLBACK (calendar_month_changed),
+										window);
+	g_signal_connect (G_OBJECT (calendar), "day_selected",
+										G_CALLBACK (calendar_day_selected),
+										window);
+	g_signal_connect (G_OBJECT (calendar), "day_selected_double_click",
+										G_CALLBACK (calendar_day_selected_double_click),
+										window);
+	window->calendar = GTK_WIDGET (calendar);
+	window->calendar_visible = TRUE;
 
-   if (window->curlog == NULL || window->calendar_visible)
-      return;
-
-   if (window->calendar_dialog == NULL)
-   {
-      GtkCalendar *calendar;
-      GtkWidget *CalendarDialog, *CalendarWidget;
-      GtkWidget *vbox;
-
-      CalendarDialog = gtk_dialog_new ();
-
-      gtk_window_set_title (GTK_WINDOW (CalendarDialog), _("Calendar"));
-      gtk_window_set_resizable (GTK_WINDOW (CalendarDialog), FALSE);
-      gtk_dialog_set_has_separator (GTK_DIALOG(CalendarDialog), FALSE);
-      gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (CalendarDialog)), 5);
-
-      gtk_dialog_add_button (GTK_DIALOG (CalendarDialog), 
-			     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-      g_signal_connect (G_OBJECT (CalendarDialog), "response",
-		        G_CALLBACK (close_calendar),
-			window);
-      g_signal_connect (G_OBJECT (CalendarDialog), "destroy",
-			G_CALLBACK (close_calendar),
-			window);
-      g_signal_connect (G_OBJECT (CalendarDialog), "delete_event",
-			G_CALLBACK (gtk_true),
-			NULL);
-
-      vbox = gtk_vbox_new (FALSE, 6);
-
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (CalendarDialog)->vbox), vbox, 
-			  TRUE, TRUE, 0);
-      gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-
-      calendar = (GtkCalendar *)gtk_calendar_new();
-
-      g_signal_connect (G_OBJECT (calendar), "month_changed",
-			G_CALLBACK (calendar_month_changed),
-			window);
-      g_signal_connect (G_OBJECT (calendar), "day_selected",
-			G_CALLBACK (calendar_day_selected),
-			window);
-      g_signal_connect (G_OBJECT (calendar), "day_selected_double_click",
-			G_CALLBACK (calendar_day_selected_double_click),
-			window);
-      gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (calendar), TRUE, TRUE, 0);
-      gtk_widget_show (GTK_WIDGET (calendar));
-
-      gtk_dialog_set_default_response (GTK_DIALOG (CalendarDialog), GTK_RESPONSE_CLOSE); 
-   
-      gtk_widget_show (vbox);
-      
-      CalendarWidget = GTK_WIDGET (calendar);
-
-      window->calendar_dialog = CalendarDialog;
-      window->calendar = CalendarWidget;
-   }
-
-   if (window->curlog->caldata == NULL)
-	   init_calendar_data (window);
-
-   window->calendar_visible = TRUE;
-   gtk_widget_show (window->calendar_dialog);
+	context = gtk_widget_get_pango_context (GTK_WIDGET(calendar));
+	fontdesc = pango_context_get_font_description (context);
+	size = pango_font_description_get_size (fontdesc) / PANGO_SCALE;
+	pango_font_description_set_size (fontdesc, (size-2)*PANGO_SCALE);
+	gtk_widget_modify_font (GTK_WIDGET (calendar), fontdesc);
 
+	gtk_widget_show (GTK_WIDGET (calendar));
 }
 
 /* ----------------------------------------------------------------------
@@ -134,8 +95,8 @@ read_marked_dates (CalendarData *data, LogviewWindow *window)
   while (mark)
     {
       if (mark->fulldate.tm_mon != month || 
-	  mark->fulldate.tm_year != year)
-	break;
+					mark->fulldate.tm_year != year)
+				break;
       gtk_calendar_mark_day (calendar, mark->fulldate.tm_mday);
       mark = mark->next;
     }
@@ -153,22 +114,30 @@ CalendarData*
 init_calendar_data (LogviewWindow *window)
 {
    CalendarData *data;
+	 
+	 if (window->curlog == NULL)
+		 return;
 
    data = window->curlog->caldata;
    if (data == NULL)
      data = (CalendarData*) malloc (sizeof (CalendarData));
 
-   if (data)
-     {
-       data->curmonthmark = window->curlog->lstats.firstmark;
-       window->curlog->caldata = data;
-
-       read_marked_dates (data, window);
-       
-       /* signal a redraw event */
-       g_signal_emit_by_name (GTK_OBJECT (window->calendar), "month_changed");
-       
-     }
+   if (data) {
+		 DateMark *mark;
+		 data->curmonthmark = window->curlog->lstats.firstmark;
+		 window->curlog->caldata = data;
+		 
+		 mark = data->curmonthmark;
+		 if (mark) {
+			 gtk_calendar_select_month (GTK_CALENDAR(window->calendar), 
+																	mark->fulldate.tm_mon,
+																	mark->fulldate.tm_year + 1900);
+			 gtk_calendar_select_day (GTK_CALENDAR(window->calendar), 
+																mark->fulldate.tm_mday);
+		 }
+		 
+		 read_marked_dates (data, window);       
+	 }
 
    return data;
 }
@@ -186,6 +155,9 @@ calendar_month_changed (GtkWidget *widget, LogviewWindow *window)
   DateMark *mark;
   gint day, month, year;
 
+	if (window->curlog == NULL)
+		return;
+
   calendar = GTK_CALENDAR (window->calendar);
   g_return_if_fail (calendar);
 
@@ -210,21 +182,24 @@ calendar_month_changed (GtkWidget *widget, LogviewWindow *window)
    DESCRIPTION:   User clicked on a calendar entry
    ---------------------------------------------------------------------- */
 
-void
+static GtkTreePath *
 calendar_day_selected (GtkWidget *widget, LogviewWindow *window)
 {
   /* find the selected day in the current logfile */
   gint day, month, year;
   GtkTreePath *path;
 
+	if (window->curlog == NULL)
+		return NULL;
+
   gtk_calendar_get_date (GTK_CALENDAR (window->calendar), &year, &month, &day);
 
   path = g_hash_table_lookup (window->curlog->date_headers,
                 DATEHASH (month, day));
   if (path != NULL)
-    {
-	    gtk_tree_view_set_cursor (GTK_TREE_VIEW(window->view), path, NULL, FALSE);
-    }
+		gtk_tree_view_set_cursor (GTK_TREE_VIEW(window->view), path, NULL, FALSE);
+
+	return path;
 }
 
 /* ----------------------------------------------------------------------
@@ -235,7 +210,10 @@ calendar_day_selected (GtkWidget *widget, LogviewWindow *window)
 void
 calendar_day_selected_double_click (GtkWidget *widget, LogviewWindow *window)
 {
-  calendar_day_selected(widget, window);
+	GtkTreePath *path;
+  path = calendar_day_selected(widget, window);
+	if (path)
+		gtk_tree_view_expand_row (GTK_TREE_VIEW (window->view), path, FALSE);
 }
 
 /* ----------------------------------------------------------------------
