@@ -22,6 +22,9 @@
 #ifdef __CYGWIN__
 #define timezonevar
 #endif
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -44,7 +47,7 @@ static gboolean file_exist (char *filename, gboolean show_error);
 
 extern GList *regexp_db;
 extern GList *actions_db;
-const char *error_main = N_("The file could not be opened");
+const char *error_main = N_("One file or more could not be opened");
 
 /*
  * -------------------
@@ -118,6 +121,10 @@ get_month (const char *str)
 		locales_monthnames = g_hash_table_new (g_str_hash, g_str_equal);
 
 	/* Try all known locales */
+	/* FIXME :
+	 * This part of the function is very slow and should not be called
+	 * on every line of a log, we should use it just once */
+
 	for (j = 0; locales != NULL && locales[j] != NULL; j++) {
 		for (i = 0; i < 12; i++) {
 			char *key = g_strdup_printf ("%s %d", locales[j], i);
@@ -184,7 +191,7 @@ file_is_zipped (char *filename)
    ---------------------------------------------------------------------- */
 
 Log *
-OpenLogFile (char *filename)
+OpenLogFile (char *filename, gboolean show_error)
 {
    Log *tlog;
    LogLine *line;
@@ -196,7 +203,7 @@ OpenLogFile (char *filename)
    GnomeVFSResult result;
    int size;
    
-   if (file_exist (filename, TRUE) == FALSE)
+   if (file_exist (filename, show_error) == FALSE)
 	   return NULL;
 
    if (file_is_zipped (filename)) {
@@ -206,7 +213,7 @@ OpenLogFile (char *filename)
 
    /* Check that the file is readable and is a logfile */
    if (!isLogFile (filename, TRUE))
-	   return NULL;
+       return NULL;
 
    result = gnome_vfs_read_entire_file (filename, &size, &buffer);
    if (result != GNOME_VFS_OK) {
@@ -220,28 +227,34 @@ OpenLogFile (char *filename)
 	   ShowErrMessage (NULL, error_main, _("Not enough memory!\n"));
 	   return NULL;
    }
-   strncpy (tlog->name, filename, sizeof (tlog->name)-1);
-   tlog->name[sizeof (tlog->name) - 1] = '\0';
+   tlog->name = g_strdup (filename);
    tlog->display_name = display_name;
+   tlog->has_date = TRUE;
    if (display_name)
 	   g_free (filename);
 
    buffer_lines = g_strsplit (buffer, "\n", -1);
+   g_free (buffer);
 
    /* count the lines */
    for (i=0; buffer_lines[i+1] != NULL; i++);
    tlog->total_lines = i;
 
-   tlog->lines = g_malloc (sizeof (*(tlog->lines)) * tlog->total_lines);
-   for (i=0; buffer_lines[i+1] != NULL; i++) {	   
-	   line = g_malloc (sizeof(**(tlog->lines)));
-	   if (!line) {
-		   ShowErrMessage (NULL, error_main, "Unable to malloc for lines[i]\n");
-		   return NULL;
-	   }
-	   ParseLine (buffer_lines[i], line);
-	   (tlog->lines)[i] = line;
-   }   
+   tlog->lines = g_new (LogLine*, tlog->total_lines);
+   for (i=0; buffer_lines[i+1]!=NULL; i++) {
+       (tlog->lines)[i] = g_new (LogLine, 1);
+       if ((tlog->lines)[i]==NULL) {
+           ShowErrMessage (NULL, error_main, _("Not enough memory!\n"));
+           return NULL;
+       }
+   }
+
+	 	 
+   for (i=0; buffer_lines[i+1] != NULL; i++) {
+	   ParseLine (buffer_lines[i], (tlog->lines)[i], tlog->has_date);
+       if ((tlog->lines)[i]->month == -1 && tlog->has_date)
+           tlog->has_date = FALSE;
+   }
 
    /* Read log stats */
    ReadLogStats (tlog, buffer_lines);
@@ -251,8 +264,25 @@ OpenLogFile (char *filename)
    tlog->date_headers = g_hash_table_new_full (NULL, NULL, NULL, 
 					       (GDestroyNotify) gtk_tree_path_free);
    tlog->first_time = TRUE;
-   return tlog;
 
+	 /* Check for older versions of the log */
+	 tlog->versions = 0;
+	 tlog->current_version = 0;
+	 tlog->parent_log = NULL;
+	 for (i=1; i<5; i++) {
+		 gchar *older_name;
+		 older_name = g_strdup_printf ("%s.%d", filename, i);
+		 tlog->older_logs[i] = OpenLogFile (older_name, FALSE);
+		 if (tlog->older_logs[i] != NULL) {
+			 tlog->older_logs[i]->parent_log = tlog;
+			 tlog->older_logs[i]->current_version = i;
+			 tlog->versions++;
+		 }
+		 else
+			 break;
+	 }
+
+   return tlog;
 }
 
 static gboolean
@@ -271,7 +301,7 @@ file_exist (char *filename, gboolean show_error)
 			   g_snprintf (buff, sizeof (buff),
 				       _("%s is not user readable. "
 					 "Either run the program as root or ask the sysadmin to "
-					 "change the permissions on the file."), filename);
+					 "change the permissions on the file.\n"), filename);
 			   break;
 		   case GNOME_VFS_ERROR_TOO_BIG:
 			   g_snprintf (buff, sizeof (buff), _("%s is too big."));
@@ -300,13 +330,11 @@ isLogFile (char *filename, gboolean show_error)
    char buff[1024];
    char **token;
    char *found_space;
-   int i;
    GnomeVFSHandle *handle;
    GnomeVFSResult result;
    GnomeVFSFileSize size;
 
-   /* Read first line and check that it has the format
-    * of a log file: Date ...    */
+   /* Read first line and check that it is text */
    result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
    if (result != GNOME_VFS_OK)
 	   return FALSE;
@@ -325,17 +353,6 @@ isLogFile (char *filename, gboolean show_error)
 	   return FALSE;
    }
    
-   token = g_strsplit (buff, " ", 1);
-   i = get_month (token[0]);
-   g_strfreev (token);
-   if (i == 12) {
-	   if (show_error) {
-		   g_snprintf (buff, sizeof (buff), _("%s not a log file."), filename);
-		   ShowErrMessage (NULL, error_main, buff);
-	   }
-	   return FALSE;
-   }
-
    /* It looks like a log file... */
    return TRUE;
 }
@@ -395,120 +412,70 @@ gchar **ReadNewLines (Log *log)
    ---------------------------------------------------------------------- */
 
 void
-ParseLine (char *buff, LogLine * line)
+ParseLine (char *buff, LogLine *line, gboolean has_date)
 {
    char *token;
    char scratch[1024];
    int i;
+   int len;
 
-   /* just copy as a whole line to be the default */
-   strncpy (line->message, buff, MAX_WIDTH);
-   line->message[MAX_WIDTH-1] = '\0';
+   /* create defaults */
+   strncpy (line->message, buff, MAX_WIDTH);   
+   len = MIN (MAX_WIDTH-1, strlen (buff));
+   line->message[len] = '\0';   
+   line->month = -1; line->date = -1;
+   line->hour = -1; line->min = -1; line->sec = -1;
+   strcpy (line->hostname, "");
+   strcpy (line->process, "");
 
+   if (!has_date)
+       return;
+
+   /* FIXME : stop using strtok, it's unrecommended. */
    token = strtok (line->message, " ");
    if (token == NULL) return;
-   /* This is not a good assumption I don't think, especially
-    * if log is internationalized
-    * -George
-   if (strlen (token) != 3)
-   {
-      strncpy (line->message, buff, MAX_WIDTH);
-      line->month = -1;
-      line->date = -1;
-      line->hour = -1;
-      line->min = -1;
-      line->sec = -1;
-      strcpy (line->hostname, "");
-      strcpy (line->process, "");
-      return;
-   }
-   */
-   i = get_month (token);
 
+   i = get_month (token);
    if (i == 12)
-   {
-      line->month = -1;
-      line->date = -1;
-      line->hour = -1;
-      line->min = -1;
-      line->sec = -1;
-      strcpy (line->hostname, "");
-      strcpy (line->process, "");
-      return;
-   }
-   line->month = i;
+       return;
+   line->month = i;	 
 
    token = strtok (NULL, " ");
    if (token != NULL)
-      line->date = (char) atoi (token);
+       line->date = (char) atoi (token);
    else
-   {
-      line->date = -1;
-      line->hour = -1;
-      line->min = -1;
-      line->sec = -1;
-      strcpy (line->hostname, "");
-      strcpy (line->process, "");
-      strcpy (line->message, "");
-      return;
-   }
+       return;
 
    token = strtok (NULL, ":");
    if (token != NULL)
-      line->hour = (char) atoi (token);
+       line->hour = (char) atoi (token);
    else
-   {
-      line->hour = -1;
-      line->min = -1;
-      line->sec = -1;
-      strcpy (line->hostname, "");
-      strcpy (line->process, "");
-      strcpy (line->message, "");
-      return;
-   }
+       return;
 
    token = strtok (NULL, ":");
    if (token != NULL)
       line->min = (char) atoi (token);
    else
-   {
-      line->min = -1;
-      line->sec = -1;
-      strcpy (line->hostname, "");
-      strcpy (line->process, "-");
-      strcpy (line->message, "");
-      return;
-   }
+       return;
 
    token = strtok (NULL, " ");
    if (token != NULL)
-      line->sec = (char) atoi (token);
+       line->sec = (char) atoi (token);
    else
-   {
-      line->sec = -1;
-      strcpy (line->hostname, "");
-      strcpy (line->process, "-");
-      strcpy (line->message, "");
-      return;
-   }
+       return;
 
    token = strtok (NULL, " ");
    if (token != NULL) {
       strncpy (line->hostname, token, MAX_HOSTNAME_WIDTH);
       line->hostname[MAX_HOSTNAME_WIDTH-1] = '\0';
-   } else {
-      line->sec = -1;
-      strcpy (line->hostname, "");
-      strcpy (line->process, "-");
-      strcpy (line->message, "");
-      return;
-   }
+   } else
+       return;
 
    token = strtok (NULL, ":\n");
    if (token != NULL)
-      strncpy (scratch, token, 254);
+       strncpy (scratch, token, 254);
    else
-      strncpy (scratch, " ", 254);
+       strncpy (scratch, " ", 254);
    scratch[254] = '\0';
    token = strtok (NULL, "\n");
 
@@ -567,6 +534,9 @@ ReadLogStats (Log *log, gchar **buffer_lines)
    log->lstats.size = info.size;
 
    /* Make sure we have at least a starting date. */
+	 if ((log->lines)[0]->month == -1)
+		 return;
+
    nl = 0;
    curdate = -1;
    while ((curdate < 0) && (buffer_lines[nl]!=NULL))
@@ -780,6 +750,10 @@ CloseLog (Log * log)
 
    g_return_if_fail (log);
 
+	 /* Close archive logs if there's some */
+	 for (i = 0; i < log->versions; i++)
+		 CloseLog (log->older_logs[i]);
+
    /* Close file - this should not be needed */
    if (log->mon_file_handle != NULL) {
 	   gnome_vfs_close (log->mon_file_handle);
